@@ -10,7 +10,7 @@ use bxcan::filter::Mask32;
 use bxcan::{Frame, StandardId};
 use cortex_m_rt::entry;
 use nb::block;
-use stm32f4xx_hal::{can::Can, pac, prelude::*};
+use stm32f4xx_hal::{pac, prelude::*};
 
 #[entry]
 fn main() -> ! {
@@ -21,14 +21,16 @@ fn main() -> ! {
     // To meet CAN clock accuracy requirements an external crystal or ceramic
     // resonator must be used. The blue pill has a 8MHz external crystal.
     // Other boards might have a crystal with another frequency or none at all.
-    rcc.cfgr.use_hse(8.mhz()).freeze();
+    rcc.cfgr.use_hse(8.MHz()).freeze();
 
     let gpiob = dp.GPIOB.split();
     let mut can1 = {
         let rx = gpiob.pb8.into_alternate::<9>();
         let tx = gpiob.pb9.into_alternate();
 
-        let can = Can::new(dp.CAN1, (tx, rx));
+        // let can = Can::new(dp.CAN1, (tx, rx));
+        // or
+        let can = dp.CAN1.can((tx, rx));
 
         bxcan::Can::builder(can)
             // APB1 (PCLK1): 8MHz, Bit rate: 500kBit/s, Sample Point 87.5%
@@ -45,7 +47,7 @@ fn main() -> ! {
         let tx = gpiob.pb13.into_alternate();
         let rx = gpiob.pb12.into_alternate();
 
-        let can = Can::new(dp.CAN2, (tx, rx));
+        let can = dp.CAN2.can((tx, rx));
 
         let can2 = bxcan::Can::builder(can)
             // APB1 (PCLK1): 8MHz, Bit rate: 500kBit/s, Sample Point 87.5%
