@@ -4,7 +4,7 @@
 //! To run this feature you can (after installing `probe-run` use:
 //!
 //! ```bash
-//! cargo run --release --example f413disco_lcd_ferris --features="stm32f413,rt,fsmc_lcd"
+//! cargo run --release --example f413disco_lcd_ferris --features="stm32f413,fsmc_lcd"
 //! ```
 
 #![no_main]
@@ -735,10 +735,7 @@ fn main() -> ! {
         };
 
         // Setup the RESET pin
-        let rst = gpiob
-            .pb13
-            .into_push_pull_output()
-            .set_speed(Speed::VeryHigh);
+        let rst = gpiob.pb13.into_push_pull_output().speed(Speed::VeryHigh);
 
         // We're not using the "tearing" signal from the display
         let mut _te = gpiob.pb14.into_floating_input();
